@@ -1,17 +1,9 @@
 ##	common/gajim.py
 ##
-## Contributors for this file:
-## - Yann Le Boulanger <asterix@lagaule.org>
-## - Nikos Kouremenos <kourem@gmail.com>
-##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2005-2006 Dimitur Kirov <dkirov@gmail.com>
+## Copyright (C) 2005-2006 Travis Shirk <travis@pobox.com>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -25,14 +17,15 @@
 
 import os
 import sys
+import tempfile
 import logging
-import mutex
+import locale
 
 import config
-
+from contacts import Contacts
 
 interface = None # The actual interface (the gtk one for the moment)
-version = '0.9.1'
+version = '0.10.1'
 config = config.Config()
 connections = {}
 verbose = False
@@ -51,34 +44,48 @@ if os.name == 'nt':
 	try:
 		# Documents and Settings\[User Name]\Application Data\Gajim
 		LOGPATH = os.path.join(os.environ['appdata'], 'Gajim', 'Logs') # deprecated
-		VCARDPATH = os.path.join(os.environ['appdata'], 'Gajim', 'Vcards')
+		VCARD_PATH = os.path.join(os.environ['appdata'], 'Gajim', 'Vcards')
+		AVATAR_PATH = os.path.join(os.environ['appdata'], 'Gajim', 'Avatars')
+		MY_EMOTS_PATH = os.path.join(os.environ['appdata'], 'Gajim', 'Emoticons')
 	except KeyError:
 		# win9x, in cwd
 		LOGPATH = 'Logs' # deprecated
-		VCARDPATH = 'Vcards'
+		VCARD_PATH = 'Vcards'
+		AVATAR_PATH = 'Avatars'
+		MY_EMOTS_PATH = 'Emoticons'
 else: # Unices
 	DATA_DIR = '../data'
 	LOGPATH = os.path.expanduser('~/.gajim/logs') # deprecated
-	VCARDPATH = os.path.expanduser('~/.gajim/vcards')
+	VCARD_PATH = os.path.expanduser('~/.gajim/vcards')
+	AVATAR_PATH = os.path.expanduser('~/.gajim/avatars')
+	MY_EMOTS_PATH = os.path.expanduser('~/.gajim/emoticons')
+
+HOME_DIR = os.path.expanduser('~')
+TMP = tempfile.gettempdir()
 
 try:
 	LOGPATH = LOGPATH.decode(sys.getfilesystemencoding())
-	VCARDPATH = VCARDPATH.decode(sys.getfilesystemencoding())
+	VCARD_PATH = VCARD_PATH.decode(sys.getfilesystemencoding())
+	TMP = TMP.decode(sys.getfilesystemencoding())
+	AVATAR_PATH = AVATAR_PATH.decode(sys.getfilesystemencoding())
+	MY_EMOTS_PATH = MY_EMOTS_PATH.decode(sys.getfilesystemencoding())
 except:
 	pass
-
-LANG = os.getenv('LANG') # en_US, fr_FR, el_GR etc..
-if LANG:
-	LANG = LANG[:2] # en, fr, el etc..
-else:
+try:
+	LANG = locale.getdefaultlocale()[0] # en_US, fr_FR, el_GR etc..
+except (ValueError, locale.Error):
+	# unknown locale, use en is better than fail
+	LANG = None
+if LANG is None:
 	LANG = 'en'
+else:
+	LANG = LANG[:2] # en, fr, el etc..
 
 last_message_time = {} # list of time of the latest incomming message
 							# {acct1: {jid1: time1, jid2: time2}, }
 encrypted_chats = {} # list of encrypted chats {acct1: [jid1, jid2], ..}
 
-contacts = {} # list of contacts {acct: {jid1: [C1, C2]}, } one Contact per resource
-gc_contacts = {} # list of contacts that are in gc {acct: {room_jid: {nick: C}}}
+contacts = Contacts()
 gc_connected = {} # tell if we are connected to the room or not {acct: {room_jid: True}}
 gc_passwords = {} # list of the pass required to enter a room {room_jid: password}
 
@@ -95,7 +102,10 @@ awaiting_events = {} # list of messages/FT reveived but not printed
 	# file-completed, file-stopped:
 		# data = file_props
 nicks = {} # list of our nick names in each account
-allow_notifications = {} # do we allow notifications for each account ?
+# should we block 'contact signed in' notifications for this account?
+# this is only for the first 30 seconds after we change our show
+# to something else than offline
+block_signed_in_notifications = {}
 con_types = {} # type of each connection (ssl, tls, tcp, ...)
 
 sleeper_state = {} # whether we pass auto away / xa or not
@@ -104,10 +114,6 @@ sleeper_state = {} # whether we pass auto away / xa or not
 #'autoaway': autoaway and use sleeper
 #'autoxa': autoxa and use sleeper
 status_before_autoaway = {}
-#queues of events from connections...
-events_for_ui = {}
-#... and its mutex
-mutex_events_for_ui = mutex.mutex()
 
 SHOW_LIST = ['offline', 'connecting', 'online', 'chat', 'away', 'xa', 'dnd',
 	'invisible']
@@ -146,52 +152,21 @@ def get_real_jid_from_fjid(account, fjid):
 	if not nick: # It's not a fake_jid, it is a real jid
 		return fjid # we return the real jid
 	real_jid = fjid
-	gcs = interface.instances[account]['gc']
-	if gcs.has_key(room_jid):
+	if interface.msg_win_mgr.get_control(room_jid, account):
 		# It's a pm, so if we have real jid it's in contact.jid
-		if not gc_contacts[account][room_jid].has_key(nick):
+		gc_contact = contacts.get_gc_contact(account, room_jid, nick)
+		if not gc_contact:
 			return
-		contact = gc_contacts[account][room_jid][nick]
-		# contact.jid is None when it's not a real jid (we don't know real jid)
-		real_jid = contact.jid
+		# gc_contact.jid is None when it's not a real jid (we don't know real jid)
+		real_jid = gc_contact.jid
 	return real_jid
 
 def get_room_from_fjid(jid):
 	return get_room_and_nick_from_fjid(jid)[0]
 
-def get_contact_instances_from_jid(account, jid):
-	''' we may have two or more resources on that jid '''
-	if jid in contacts[account]:
-		contacts_instances = contacts[account][jid]
-		return contacts_instances
-
-def get_first_contact_instance_from_jid(account, jid):
-	contact = None
-	if jid in contacts[account]:
-		contact = contacts[account][jid][0]
-	else: # it's fake jid
-		#FIXME: problem see comment in next line
-		room, nick = \
-			get_room_and_nick_from_fjid(jid) # if we ban/kick we now real jid
-		if gc_contacts[account].has_key(room) and \
-		nick in gc_contacts[account][room]:
-			contact = gc_contacts[account][room][nick]
-	return contact
-
-def get_contact_instance_with_highest_priority(account, jid):
-	contact_instances = contacts[account][jid]
-	return get_highest_prio_contact_from_contacts(contact_instances)
-
 def get_contact_name_from_jid(account, jid):
-	return contacts[account][jid][0].name
-
-def get_highest_prio_contact_from_contacts(contacts):
-	prim_contact = None # primary contact
-	for contact in contacts:
-		if prim_contact == None or int(contact.priority) > \
-			int(prim_contact.priority):
-			prim_contact = contact
-	return prim_contact
+	c = contacts.get_first_contact_from_jid(account, jid)
+	return c.name
 
 def get_jid_without_resource(jid):
 	return jid.split('/')[0]
@@ -218,61 +193,82 @@ def get_resource_from_jid(jid):
 '''
 
 def get_number_of_accounts():
+	'''returns the number of ALL accounts'''
 	return len(connections.keys())
 
+def get_number_of_connected_accounts(accounts_list = None):
+	'''returns the number of CONNECTED accounts
+	you can optionally pass an accounts_list
+	and if you do those will be checked, else all will be checked'''
+	connected_accounts = 0
+	if accounts_list is None:
+		accounts = connections.keys()
+	else:
+		accounts = accounts_list
+	for acct in accounts:
+		if connections[acct].connected > 1:
+			connected_accounts = connected_accounts + 1
+	return connected_accounts
+
 def get_transport_name_from_jid(jid, use_config_setting = True):
-	'''returns 'aim', 'gg', 'irc' etc'''
+	'''returns 'aim', 'gg', 'irc' etc
+	if JID is not from transport returns None'''
 	#FIXME: jid can be None! one TB I saw had this problem:
 	# in the code block # it is a groupchat presence in handle_event_notify
 	# jid was None. Yann why?
 	if not jid or (use_config_setting and not config.get('use_transports_iconsets')):
 		return
-	host = jid.split('@')[-1]
-	if host.startswith('aim'):
+	
+	host = get_server_from_jid(jid)
+	# host is now f.e. icq.foo.org or just icq (sometimes on hacky transports)
+	host_splitted = host.split('.')
+	if len(host_splitted) != 0:
+		# now we support both 'icq.' and 'icq' but not icqsucks.org
+		host = host_splitted[0]
+
+	if host == 'aim':
 		return 'aim'
-	elif host.startswith('gg'):
+	elif host == 'gg':
 		return 'gadugadu'
-	elif host.startswith('irc'):
+	elif host == 'irc':
 		return 'irc'
-	# abc@icqsucks.org will match as ICQ, but what to do..
-	elif host.startswith('icq'):
+	elif host == 'icq':
 		return 'icq'
-	elif host.startswith('msn'):
+	elif host == 'jit':
+		return 'icq'
+	elif host == 'msn':
 		return 'msn'
-	elif host.startswith('sms'):
+	elif host == 'sms':
 		return 'sms'
-	elif host.startswith('tlen'):
+	elif host == 'tlen':
 		return 'tlen'
-	elif host.startswith('weather'):
+	elif host == 'weather':
 		return 'weather'
-	elif host.startswith('yahoo'):
+	elif host == 'yahoo':
 		return 'yahoo'
 	else:
 		return None
 
 def jid_is_transport(jid):
-	aim = jid.startswith('aim')
-	gg = jid.startswith('gg') # gadugadu
-	irc = jid.startswith('irc')
-	icq = jid.startswith('icq')
-	msn = jid.startswith('msn')
-	sms = jid.startswith('sms')
-	tlen = jid.startswith('tlen')
-	yahoo = jid.startswith('yahoo')
-
-	if aim or gg or irc or icq or msn or sms or yahoo or tlen:
-		is_transport = True
-	else:
-		is_transport = False
-
-	return is_transport
+	# if not '@' or '@' starts the jid then it is transport
+	if jid.find('@') <= 0:
+		return True
+	return False
 
 def get_jid_from_account(account_name):
+	'''return the jid we use in the given account'''
 	name = config.get_per('accounts', account_name, 'name')
 	hostname = config.get_per('accounts', account_name, 'hostname')
 	jid = name + '@' + hostname
 	return jid
 
+def get_our_jids():
+	'''returns a list of the jids we use in our accounts'''
+	our_jids = list()
+	for account in contacts.get_accounts():
+		our_jids.append(get_jid_from_account(account))
+	return our_jids
+
 def get_hostname_from_account(account_name, use_srv = False):
 	'''returns hostname (if custom hostname is used, that is returned)'''
 	if use_srv and connections[account_name].connected_hostname:
@@ -292,19 +288,21 @@ def get_first_event(account, jid, typ = None):
 		if ev[0] == typ:
 			return ev
 	return None
+	
+def get_notification_image_prefix(jid):
+	'''returns the prefix for the notification images'''
+	transport_name = get_transport_name_from_jid(jid)
+	if transport_name in ('aim', 'icq', 'msn', 'yahoo'):
+		prefix = transport_name
+	else:
+		prefix = 'jabber'
+	return prefix
 
-def popup_window(account):
-	autopopup = config.get('autopopup')
-	autopopupaway = config.get('autopopupaway')
-	if autopopup and (autopopupaway or connections[account].connected > 3):
-		return True
-	return False
-
-def show_notification(account):
-	if config.get('notify_on_new_message'):
-		# check OUR status and if we allow notifications for that status
-		if config.get('autopopupaway'): # always show notification
-			return True
-		if connections[account].connected in (2, 3): # we're online or chat
-			return True
-	return False
+def get_name_from_jid(account, jid):
+	'''returns from JID's shown name and if no contact returns jids'''
+	contact = contacts.get_first_contact_from_jid(account, jid)
+	if contact:
+		actor = contact.get_shown_name()
+	else:
+		actor = jid
+	return actor
