@@ -33,6 +33,7 @@ import tooltips
 import dialogs
 import locale
 
+import gtkgui_helpers
 from common import gajim
 from common import helpers
 from common import i18n
@@ -43,8 +44,6 @@ APP = i18n.APP
 gtk.glade.bindtextdomain(APP, i18n.DIR)
 gtk.glade.textdomain(APP)
 
-GTKGUI_GLADE = 'gtkgui.glade'
-
 class ConversationTextview:
 	'''Class for the conversation textview (where user reads already said messages)
 	for chat/groupchat windows'''
@@ -160,7 +159,10 @@ class ConversationTextview:
 	def scroll_to_end(self):
 		parent = self.tv.get_parent()
 		buffer = self.tv.get_buffer()
-		self.tv.scroll_to_mark(buffer.get_mark('end'), 0, True, 0, 1)
+		end_mark = buffer.get_mark('end')
+		if not end_mark:
+			return False
+		self.tv.scroll_to_mark(end_mark, 0, True, 0, 1)
 		adjustment = parent.get_hadjustment()
 		adjustment.set_value(0)
 		return False # when called in an idle_add, just do it once
@@ -370,7 +372,7 @@ class ConversationTextview:
 		dialogs.AddNewContactWindow(self.account, jid)
 
 	def make_link_menu(self, event, kind, text):
-		xml = gtk.glade.XML(GTKGUI_GLADE, 'chat_context_menu', APP)
+		xml = gtkgui_helpers.get_glade('chat_context_menu.glade')
 		menu = xml.get_widget('chat_context_menu')
 		childs = menu.get_children()
 		if kind == 'url':
