@@ -1,18 +1,11 @@
 ##	filetransfers_window.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <kourem@gmail.com>
-##	- Dimitur Kirov <dkirov@gmail.com>
-##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
+## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2005
 ##                    Dimitur Kirov <dkirov@gmail.com>
 ##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2004-2005 Vincent Hanquez <tab@snarc.org>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -29,7 +22,6 @@ import gtk.glade
 import gobject
 import pango
 import os
-import sys
 import time
 
 import gtkgui_helpers
@@ -45,8 +37,6 @@ APP = i18n.APP
 gtk.glade.bindtextdomain (APP, i18n.DIR)
 gtk.glade.textdomain (APP)
 
-GTKGUI_GLADE = 'gtkgui.glade'
-
 C_IMAGE = 0
 C_LABELS = 1
 C_FILE = 2
@@ -60,7 +50,7 @@ class FileTransfersWindow:
 	def __init__(self):
 		self.files_props = {'r' : {}, 's': {}}
 		self.height_diff = 0
-		self.xml = gtk.glade.XML(GTKGUI_GLADE, 'file_transfers_window', APP)
+		self.xml = gtkgui_helpers.get_glade('filetransfers.glade')
 		self.window = self.xml.get_widget('file_transfers_window')
 		self.tree = self.xml.get_widget('transfers_list')
 		self.cancel_button = self.xml.get_widget('cancel_button')
@@ -128,18 +118,15 @@ class FileTransfersWindow:
 		self.tree.get_selection().set_mode(gtk.SELECTION_SINGLE)
 		self.tree.get_selection().connect('changed', self.selection_changed)
 		self.tooltip = tooltips.FileTransfersTooltip()
-		self.xml.signal_autoconnect(self)
-		popup_xml = gtk.glade.XML(GTKGUI_GLADE, 'file_transfers_menu',
-			APP)
-		self.file_transfers_menu = popup_xml.get_widget('file_transfers_menu')
-		self.open_folder_menuitem = popup_xml.get_widget('open_folder_menuitem')
-		self.cancel_menuitem = popup_xml.get_widget('cancel_menuitem')
-		self.pause_menuitem = popup_xml.get_widget('pause_menuitem')
-		self.continue_menuitem = popup_xml.get_widget('continue_menuitem')
+		self.file_transfers_menu = self.xml.get_widget('file_transfers_menu')
+		self.open_folder_menuitem = self.xml.get_widget('open_folder_menuitem')
+		self.cancel_menuitem = self.xml.get_widget('cancel_menuitem')
+		self.pause_menuitem = self.xml.get_widget('pause_menuitem')
+		self.continue_menuitem = self.xml.get_widget('continue_menuitem')
 		self.continue_menuitem.hide()
 		self.continue_menuitem.set_no_show_all(True)
-		self.remove_menuitem = popup_xml.get_widget('remove_menuitem')
-		popup_xml.signal_autoconnect(self)
+		self.remove_menuitem = self.xml.get_widget('remove_menuitem')
+		self.xml.signal_autoconnect(self)
 		
 	def find_transfer_by_jid(self, account, jid):
 		''' find all transfers with peer 'jid' that belong to 'account' '''
@@ -166,20 +153,29 @@ class FileTransfersWindow:
 		''' show a dialog saying that file (file_props) has been transferred'''
 		self.window.present()
 		self.window.window.focus()
+
+		def on_open(widget, file_props):
+			self.dialog.destroy()
+			if not file_props.has_key('file-name'):
+				return
+			(path, file) = os.path.split(file_props['file-name'])
+			if os.path.exists(path) and os.path.isdir(path):
+				helpers.launch_file_manager(path)
+			self.tree.get_selection().unselect_all()
+
 		if file_props['type'] == 'r':
 			# file path is used below in 'Save in'
 			(file_path, file_name) = os.path.split(file_props['file-name'])
 		else:
 			file_name = file_props['name']
-		sectext = '\t' + _('Filename: %s') % \
-			gtkgui_helpers.escape_for_pango_markup(file_name)
+		sectext = '\t' + _('Filename: %s') % file_name
 		sectext += '\n\t' + _('Size: %s') % \
 		helpers.convert_bytes(file_props['size'])
 		if file_props['type'] == 'r':
 			jid = unicode(file_props['sender']).split('/')[0]
-			sender_name = gajim.get_first_contact_instance_from_jid( 
-				file_props['tt_account'], jid).name
-			sender = gtkgui_helpers.escape_for_pango_markup(sender_name)
+			sender_name = gajim.contacts.get_first_contact_from_jid( 
+				file_props['tt_account'], jid).get_shown_name()
+			sender = sender_name
 		else:
 			#You is a reply of who sent a file
 			sender = _('You')
@@ -187,32 +183,27 @@ class FileTransfersWindow:
 		sectext += '\n\t' +_('Recipient: ')
 		if file_props['type'] == 's':
 			jid = unicode(file_props['receiver']).split('/')[0]
-			receiver_name = gajim.get_first_contact_instance_from_jid( 
-				file_props['tt_account'], jid).name
-			recipient = gtkgui_helpers.escape_for_pango_markup(receiver_name)
+			receiver_name = gajim.contacts.get_first_contact_from_jid( 
+				file_props['tt_account'], jid).get_shown_name()
+			recipient = receiver_name
 		else:
 			#You is a reply of who received a file
 			recipient = _('You')
 		sectext += recipient
 		if file_props['type'] == 'r':
-			sectext += '\n\t' +_('Saved in: %s') % \
-				gtkgui_helpers.escape_for_pango_markup(file_path)
-		dialog = dialogs.HigDialog(None, gtk.MESSAGE_INFO, gtk.BUTTONS_NONE, 
+			sectext += '\n\t' +_('Saved in: %s') % file_path
+		self.dialog = dialogs.HigDialog(None, gtk.MESSAGE_INFO, gtk.BUTTONS_NONE, 
 				_('File transfer completed'), sectext)
 		if file_props['type'] == 'r':
-			dialog.add_buttons(_('_Open Containing Folder'), gtk.RESPONSE_ACCEPT)
-		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK)
-		dialog.show_all()
-		response = dialog.run()
-		dialog.destroy()
-		if response == gtk.RESPONSE_ACCEPT:
-			if not file_props.has_key('file-name'):
-				return
-			(path, file) = os.path.split(file_props['file-name'])
-			if os.path.exists(path) and os.path.isdir(path):
-				helpers.launch_file_manager(path)
-			self.tree.get_selection().unselect_all()
-		
+			button = gtk.Button(_('_Open Containing Folder'))
+			button.connect('clicked', on_open, file_props)
+			self.dialog.action_area.pack_start(button)
+		ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
+		def on_ok(widget):
+			self.dialog.destroy()
+		ok_button.connect('clicked', on_ok)
+		self.dialog.show_all()
+
 	def show_request_error(self, file_props):
 		''' show error dialog to the recipient saying that transfer 
 		has been canceled'''
@@ -220,7 +211,7 @@ class FileTransfersWindow:
 		self.window.window.focus()
 		dialogs.InformationDialog(_('File transfer canceled'), _('Connection with peer cannot be established.'))
 		self.tree.get_selection().unselect_all()
-		
+
 	def show_send_error(self, file_props):
 		''' show error dialog to the sender saying that transfer 
 		has been canceled'''
@@ -229,7 +220,7 @@ class FileTransfersWindow:
 		dialogs.InformationDialog(_('File transfer canceled'),
 _('Connection with peer cannot be established.'))
 		self.tree.get_selection().unselect_all()
-	
+
 	def show_stopped(self, jid, file_props):
 		self.window.present()
 		self.window.window.focus()
@@ -237,59 +228,50 @@ _('Connection with peer cannot be established.'))
 			file_name = os.path.basename(file_props['file-name'])
 		else:
 			file_name = file_props['name']
-		sectext = '\t' + _('Filename: %s') % \
-			gtkgui_helpers.escape_for_pango_markup(file_name)
-		sectext += '\n\t' + _('Sender: %s') % \
-			gtkgui_helpers.escape_for_pango_markup(jid)
+		sectext = '\t' + _('Filename: %s') % file_name
+		sectext += '\n\t' + _('Sender: %s') % jid
 		dialogs.ErrorDialog(_('File transfer stopped by the contact of the other side'), \
-			sectext).get_response()
+			sectext)
 		self.tree.get_selection().unselect_all()
-		
+
 	def show_file_send_request(self, account, contact):
-		last_send_dir = gajim.config.get('last_send_dir')
-		dialog = gtk.FileChooserDialog(title=_('Choose File to Send...'), 
-			action=gtk.FILE_CHOOSER_ACTION_OPEN, 
-			buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
-		butt = dialog.add_button(_('Send'), gtk.RESPONSE_OK)
-		butt.set_use_stock(True)
-		dialog.set_default_response(gtk.RESPONSE_OK)
-		dialog.set_select_multiple(True) # we can select many files to send
-		if last_send_dir and os.path.isdir(last_send_dir):
-			dialog.set_current_folder(last_send_dir)
-		else:
-			home_dir = os.path.expanduser('~')
-			dialog.set_current_folder(home_dir)
-		file_props = {}
-		while True:
-			response = dialog.run()
-			if response == gtk.RESPONSE_OK:
-				file_dir = None
-				files_path_list = dialog.get_filenames()
-				for file_path in files_path_list:
-					file_path = file_path.decode('utf8')
-					if self.send_file(account, contact, file_path) and file_dir is None:
-						file_dir = os.path.dirname(file_path)
-				if file_dir:
-					gajim.config.set('last_send_dir', file_dir)
-					dialog.destroy()
-					break
-			else:
-				dialog.destroy()
-				break
-		
+		def on_ok(widget):
+			file_dir = None
+			files_path_list = self.dialog.get_filenames()
+			files_path_list = gtkgui_helpers.decode_filechooser_file_paths(
+				files_path_list)
+			for file_path in files_path_list:
+				if self.send_file(account, contact, file_path) and file_dir is None:
+					file_dir = os.path.dirname(file_path)
+			if file_dir:
+				gajim.config.set('last_send_dir', file_dir)
+				self.dialog.destroy()
+
+		self.dialog = dialogs.FileChooserDialog(_('Choose File to Send...'), 
+			gtk.FILE_CHOOSER_ACTION_OPEN, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL),
+			gtk.RESPONSE_OK,
+			True, # select multiple true as we can select many files to send
+			gajim.config.get('last_send_dir'),
+			)
+
+		btn = self.dialog.add_button(_('_Send'), gtk.RESPONSE_OK)
+		btn.set_use_stock(True) # FIXME: add send icon to this button (JUMP_TO)
+		btn.connect('clicked', on_ok)
+
 	def send_file(self, account, contact, file_path):
 		''' start the real transfer(upload) of the file '''
 		if gtkgui_helpers.file_is_locked(file_path):
 			pritext = _('Gajim cannot access this file')
 			sextext = _('This file is being used by another process.')
-			dialogs.ErrorDialog(pritext, sextext).get_response()
+			dialogs.ErrorDialog(pritext, sextext)
 			return
-		
+
 		if isinstance(contact, str):
 			if contact.find('/') == -1:
 				return
 			(jid, resource) = contact.split('/', 1)
-			contact = gajim.Contact(jid = jid, resource = resource)
+			contact = gajim.contacts.create_contact(jid = jid,
+				resource = resource)
 		(file_dir, file_name) = os.path.split(file_path)
 		file_props = self.get_send_file_props(account, contact, 
 				file_path, file_name)
@@ -298,13 +280,20 @@ _('Connection with peer cannot be established.'))
 		self.add_transfer(account, contact, file_props)
 		gajim.connections[account].send_file_request(file_props)
 		return True
-	
+
+	def _start_receive(self, file_path, account, contact, file_props):
+		file_dir = os.path.dirname(file_path)
+		if file_dir:
+			gajim.config.set('last_save_dir', file_dir)
+		file_props['file-name'] = file_path
+		self.add_transfer(account, contact, file_props)
+		gajim.connections[account].send_file_approval(file_props)
+
 	def show_file_request(self, account, contact, file_props):
 		''' show dialog asking for comfirmation and store location of new
 		file requested by a contact'''
 		if file_props is None or not file_props.has_key('name'):
 			return
-		last_save_dir = gajim.config.get('last_save_dir')
 		sec_text = '\t' + _('File: %s') % file_props['name']
 		if file_props.has_key('size'):
 			sec_text += '\n\t' + _('Size: %s') % \
@@ -314,47 +303,62 @@ _('Connection with peer cannot be established.'))
 		if file_props.has_key('desc'):
 			sec_text += '\n\t' + _('Description: %s') % file_props['desc']
 		prim_text = _('%s wants to send you a file:') % contact.jid
-		dialog = dialogs.ConfirmationDialog(prim_text, sec_text)
-		if dialog.get_response() == gtk.RESPONSE_OK:
-			dialog = gtk.FileChooserDialog(title=_('Save File as...'), 
-				action=gtk.FILE_CHOOSER_ACTION_SAVE, 
-				buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
-				gtk.STOCK_SAVE, gtk.RESPONSE_OK))
-			dialog.set_current_name(file_props['name'])
-			dialog.set_default_response(gtk.RESPONSE_OK)
-			gtk28 = False
-			if gtk.gtk_version >= (2, 8, 0) and gtk.pygtk_version >= (2, 8, 0):
-				dialog.props.do_overwrite_confirmation = True
-				gtk28 = True
-			if last_save_dir and os.path.isdir(last_save_dir):
-				dialog.set_current_folder(last_save_dir)
-			else:
-				home_dir = os.path.expanduser('~')
-				dialog.set_current_folder(home_dir)
-			while True:
-				response = dialog.run()
-				if response == gtk.RESPONSE_OK:
-					file_path = dialog.get_filename()
-					file_path = file_path.decode('utf-8')
-					if not gtk28 and os.path.exists(file_path):
-						primtext = _('This file already exists')
-						sectext = _('Would you like to overwrite it?')
-						dialog2 = dialogs.ConfirmationDialog(primtext, sectext)
-						if dialog2.get_response() != gtk.RESPONSE_OK:
-							continue
-					file_dir = os.path.dirname(file_path)
-					if file_dir:
-						gajim.config.set('last_save_dir', file_dir)
-					file_props['file-name'] = file_path
-					self.add_transfer(account, contact, file_props)
-					gajim.connections[account].send_file_approval(file_props)
-				else:
-					gajim.connections[account].send_file_rejection(file_props)
-				dialog.destroy()
-				break
-		else:
+		dialog, dialog2 = None, None
+
+		def on_response_ok(widget, account, contact, file_props):
+			dialog.destroy()
+
+			def on_ok(widget, account, contact, file_props):
+				file_path = dialog2.get_filename()
+				file_path = gtkgui_helpers.decode_filechooser_file_paths(
+					(file_path,))[0]
+				if os.path.exists(file_path):
+					stat = os.stat(file_path)
+					dl_size = stat.st_size
+					file_size = file_props['size']
+					dl_finished = dl_size >= file_size
+					dialog = dialogs.FTOverwriteConfirmationDialog(
+						_('This file already exists'), _('What do you want to do?'),
+						not dl_finished)
+					dialog.set_transient_for(dialog2)
+					dialog.set_destroy_with_parent(True)
+					response = dialog.get_response()
+					if response < 0:
+						return
+					elif response == 100:
+						file_props['offset'] = dl_size
+				dialog2.destroy()
+				self._start_receive(file_path, account, contact, file_props)
+
+			def on_cancel(widget, account, contact, file_props):
+				dialog2.destroy()
+				gajim.connections[account].send_file_rejection(file_props)
+
+			dialog2 = dialogs.FileChooserDialog(
+				title_text = _('Save File as...'), 
+				action = gtk.FILE_CHOOSER_ACTION_SAVE, 
+				buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
+				gtk.STOCK_SAVE, gtk.RESPONSE_OK),
+				default_response = gtk.RESPONSE_OK,
+				current_folder = gajim.config.get('last_save_dir'),
+				on_response_ok = (on_ok, account, contact, file_props),
+				on_response_cancel = (on_cancel, account, contact, file_props))
+
+			dialog2.set_current_name(file_props['name'])
+			dialog2.connect('delete-event', lambda widget, event:
+				on_cancel(widget, account, contact, file_props))
+
+		def on_response_cancel(widget, account, file_props):
+			dialog.destroy()
 			gajim.connections[account].send_file_rejection(file_props)
-	
+
+		dialog = dialogs.NonModalConfirmationDialog(prim_text, sec_text,
+			on_response_ok = (on_response_ok, account, contact, file_props),
+			on_response_cancel = (on_response_cancel, account, file_props))
+		dialog.connect('delete-event', lambda widget, event: 
+			on_response_cancel(widget, account, file_props))
+		dialog.popup()
+
 	def set_images(self):
 		''' create pixbufs for status images in transfer rows'''
 		self.images = {}
@@ -430,6 +434,20 @@ _('Connection with peer cannot be established.'))
 			if gajim.connections.has_key(account):
 				# there is a connection to the account
 				gajim.connections[account].remove_transfer(file_props)
+			if file_props['type'] == 'r': # we receive a file
+				other = file_props['sender']
+			else: # we send a file
+				other = file_props['receiver']
+			if isinstance(other, unicode):
+				jid = gajim.get_jid_without_resource(other)
+			else: # It's a Contact instance
+				jid = other.jid
+			if gajim.awaiting_events[account].has_key(jid):
+				for event in gajim.awaiting_events[account][jid]:
+					if event[0] in ('file-error', 'file-completed',
+						'file-request-error', 'file-send-error', 'file-stopped') and \
+						event[1]['sid'] == file_props['sid']:
+						gajim.interface.remove_event(account, jid, event)
 		del(self.files_props[sid[0]][sid[1:]])
 		del(file_props)
 		
@@ -455,6 +473,9 @@ _('Connection with peer cannot be established.'))
 			# Kb/s
 			
 			# remaining time
+			if file_props.has_key('offset') and file_props['offset']:
+				transfered_size -= file_props['offset'] 
+				full_size -= file_props['offset']
 			eta, speed = self._get_eta_and_speed(full_size, transfered_size, 
 				file_props['elapsed-time'])
 			
@@ -481,7 +502,7 @@ _('Connection with peer cannot be established.'))
 			if file_props.has_key('connected') and file_props['connected'] == False:
 				status = 'stop'
 			self.model.set(iter, 0, self.images[status])
-			if percent == 100:
+			if transfered_size == full_size:
 				self.set_status(typ, sid, 'ok')
 	
 	def get_iter_by_sid(self, typ, sid):
@@ -493,33 +514,23 @@ _('Connection with peer cannot be established.'))
 				return iter
 			iter = self.model.iter_next(iter)
 	
-	def get_sid(self):
-		''' create random string of length 16'''
-		rng = range(65, 90)
-		rng.extend(range(48, 57))
-		char_sequence = map(lambda e:chr(e), rng)
-		from random import sample
-		return reduce(lambda e1, e2: e1 + e2, 
-				sample(char_sequence, 16))
-	
 	def get_send_file_props(self, account, contact, file_path, file_name):
 		''' create new file_props dict and set initial file transfer 
 		properties in it'''
 		file_props = {'file-name' : file_path, 'name' : file_name, 
 			'type' : 's'}
 		if os.path.isfile(file_path):
-			
 			stat = os.stat(file_path)
 		else:
-			dialogs.ErrorDialog(_('Invalid File'), _('File: ')  + file_path).get_response()
+			dialogs.ErrorDialog(_('Invalid File'), _('File: ')  + file_path)
 			return None
 		if stat[6] == 0:
 			dialogs.ErrorDialog(_('Invalid File'), 
-			_('It is not possible to send empty files')).get_response()
+			_('It is not possible to send empty files'))
 			return None
 		file_props['elapsed-time'] = 0
 		file_props['size'] = unicode(stat[6])
-		file_props['sid'] = self.get_sid()
+		file_props['sid'] = helpers.get_random_string_16()
 		file_props['completed'] = False
 		file_props['started'] = False
 		file_props['sender'] = account
@@ -545,8 +556,8 @@ _('Connection with peer cannot be established.'))
 			(file_path, file_name) = os.path.split(file_props['file-name'])
 		else:
 			file_name = file_props['name']
-		text_props = gtkgui_helpers.escape_for_pango_markup(file_name) + '\n'
-		text_props += gtkgui_helpers.escape_for_pango_markup(contact.name)
+		text_props = file_name + '\n'
+		text_props += contact.get_shown_name()
 		self.model.set(iter, 1, text_labels, 2, text_props, C_SID,
 			file_props['type'] + file_props['sid'])
 		self.set_progress(file_props['type'], file_props['sid'], 0, iter)
@@ -758,6 +769,7 @@ _('Connection with peer cannot be established.'))
 			types = {'r' : 'download', 's' : 'upload'}
 			self.set_status(file_props['type'], file_props['sid'], types[sid[0]])
 			self.toggle_pause_continue(True)
+			file_props['continue_cb']()
 		elif self.is_transfer_active(file_props):
 			file_props['paused'] = True
 			self.set_status(file_props['type'], file_props['sid'], 'pause')
@@ -785,16 +797,18 @@ _('Connection with peer cannot be established.'))
 		pointer = self.tree.get_pointer()
 		props = self.tree.get_path_at_pos(pointer[0], 
 			pointer[1] - self.height_diff)
+		# check if the current pointer is at the same path
+		# as it was before setting the timeout
 		if props and self.tooltip.id == props[0]:
-			# check if the current pointer is at the same path
-			# as it was before setting the timeout
 			iter = self.model.get_iter(props[0])
 			sid = self.model[iter][C_SID].decode('utf-8')
 			file_props = self.files_props[sid[0]][sid[1:]]
+			# bounding rectangle of coordinates for the cell within the treeview
 			rect =  self.tree.get_cell_area(props[0],props[1])
+			# position of the treeview on the screen
 			position = widget.window.get_origin()
-			self.tooltip.show_tooltip(file_props , (pointer[0], rect.height ), 
-				(position[0], position[1] + rect.y + self.height_diff))
+			self.tooltip.show_tooltip(file_props , rect.height, 
+								position[1] + rect.y + self.height_diff)
 		else:
 			self.tooltip.hide_tooltip()
 	
@@ -817,19 +831,10 @@ _('Connection with peer cannot be established.'))
 			path = self.model.get_path(iter)
 		self.set_buttons_sensitive(path, True)
 		
-		event_button = self.get_possible_button_event(event)
+		event_button = gtkgui_helpers.get_possible_button_event(event)
+		self.file_transfers_menu.show_all()
 		self.file_transfers_menu.popup(None, self.tree, None, 
 			event_button, event.time)
-		self.file_transfers_menu.show_all()
-	
-	def get_possible_button_event(self, event):
-		'''mouse or keyboard caused the event?'''
-		if event.type == gtk.gdk.KEY_PRESS:
-			event_button = 0 # no event.button so pass 0
-		else: # BUTTON_PRESS event, so pass event.button
-			event_button = event.button
-		
-		return event_button
 	
 	def on_transfers_list_key_press_event(self, widget, event):
 		'''when a key is pressed in the treeviews'''
