@@ -26,13 +26,8 @@
 ##
 
 import gobject
-import gtk
 import os
-import sys
 
-import systray
-
-from common import exceptions
 from common import gajim
 from common import helpers
 from time import time
@@ -56,9 +51,75 @@ INTERFACE = 'org.gajim.dbus.RemoteInterface'
 OBJ_PATH = '/org/gajim/dbus/RemoteObject'
 SERVICE = 'org.gajim.dbus'
 
+# type mapping, it is different in each version
+ident = lambda e: e
+if dbus_support.version[1] >= 43:
+	# in most cases it is a utf-8 string
+	DBUS_STRING = dbus.String
+	
+	# general type (for use in dicts, 
+	#   where all values should have the same type)
+	DBUS_VARIANT = dbus.Variant
+	DBUS_BOOLEAN = dbus.Boolean
+	DBUS_DOUBLE = dbus.Double
+	DBUS_INT32 = dbus.Int32
+	# dictionary with string key and binary value
+	DBUS_DICT_SV = lambda : dbus.Dictionary({}, signature="sv")
+	# dictionary with string key and value
+	DBUS_DICT_SS = lambda : dbus.Dictionary({}, signature="ss")
+	# empty type
+	DBUS_NONE = lambda : dbus.Variant(0)
+	
+else: # 33, 35, 36
+	DBUS_DICT_SV = lambda : {}
+	DBUS_DICT_SS = lambda : {}
+	DBUS_STRING = lambda e: unicode(e).encode('utf-8')
+	# this is the only way to return lists and dicts of mixed types
+	DBUS_VARIANT = lambda e: (isinstance(e, (str, unicode)) and \
+								DBUS_STRING(e)) or repr(e)
+	DBUS_NONE = lambda : ''
+	if dbus_support.version[1] >= 41: # 35, 36
+		DBUS_BOOLEAN = dbus.Boolean
+		DBUS_DOUBLE = dbus.Double
+		DBUS_INT32 = dbus.Int32
+	else: # 33
+		DBUS_BOOLEAN = ident
+		DBUS_INT32 = ident
+		DBUS_DOUBLE = ident
+
 STATUS_LIST = ['offline', 'connecting', 'online', 'chat', 'away', 'xa', 'dnd',
 	'invisible']
 
+def get_dbus_struct(obj):
+	''' recursively go through all the items and replace
+	them with their casted dbus equivalents
+	'''
+	if obj is None:
+		return DBUS_NONE()
+	if isinstance(obj, (unicode, str)):
+		return DBUS_STRING(obj)
+	if isinstance(obj, int):
+		return DBUS_INT32(obj)
+	if isinstance(obj, float):
+		return DBUS_DOUBLE(obj)
+	if isinstance(obj, bool):
+		return DBUS_BOOLEAN(obj)
+	if isinstance(obj, (list, tuple)):
+		result = [DBUS_VARIANT(get_dbus_struct(i)) for i in obj]
+		if result == []:
+			return DBUS_NONE()
+		return result
+	if isinstance(obj, dict):
+		result = DBUS_DICT_SV()
+		for key, value in obj.items():
+			result[DBUS_STRING(key)] = DBUS_VARIANT(get_dbus_struct(
+													value))
+		if result == {}:
+			return DBUS_NONE()
+		return result
+	# unknown type
+	return DBUS_NONE() 
+
 class Remote:
 	def __init__(self):
 		self.signal_object = None
@@ -73,8 +134,9 @@ class Remote:
 
 	def raise_signal(self, signal, arg):
 		if self.signal_object:
-			self.signal_object.raise_signal(signal, repr(arg))
-		
+			self.signal_object.raise_signal(signal, 
+										get_dbus_struct(arg))
+
 
 class SignalObject(DbusPrototype):
 	''' Local object definition for /org/gajim/dbus/RemoteObject. This doc must 
@@ -95,6 +157,7 @@ class SignalObject(DbusPrototype):
 				self.show_next_unread,
 				self.list_contacts,
 				self.list_accounts,
+				self.account_info,
 				self.change_status,
 				self.open_chat,
 				self.send_message,
@@ -107,71 +170,86 @@ class SignalObject(DbusPrototype):
 				self.add_contact,
 				self.remove_contact,
 				self.get_status,
+				self.get_status_message,
 			])
 
 	def raise_signal(self, signal, arg):
 		''' raise a signal, with a single string message '''
-		if dbus_support.version[1] >= 30:
-			from dbus import dbus_bindings
-			message = dbus_bindings.Signal(OBJ_PATH, INTERFACE, signal)
-			i = message.get_iter(True)
-			i.append(arg)
-			self._connection.send(message)
-		else:
-			self.emit_signal(INTERFACE, signal, arg)
-
+		from dbus import dbus_bindings
+		message = dbus_bindings.Signal(OBJ_PATH, INTERFACE, signal)
+		i = message.get_iter(True)
+		i.append(arg)
+		self._connection.send(message)
 	
-	# signals 
-	def VcardInfo(self, *vcard):
-		pass
-
 	def get_status(self, *args):
 		'''get_status(account = None)
 		returns status (show to be exact) which is the global one
 		unless account is given'''
 		account = self._get_real_arguments(args, 1)[0]
-		accounts = gajim.contacts.keys()
+		accounts = gajim.contacts.get_accounts()
 		if not account:
 			# If user did not ask for account, returns the global status
 			return helpers.get_global_show()
 		# return show for the given account
 		index = gajim.connections[account].connected
-		return STATUS_LIST[index]
+		return DBUS_STRING(STATUS_LIST[index])
+	
+	def get_status_message(self, *args):
+		'''get_status(account = None)
+		returns status which is the global one
+		unless account is given'''
+		account = self._get_real_arguments(args, 1)[0]
+		accounts = gajim.contacts.get_accounts()
+		if not account:
+			# If user did not ask for account, returns the global status
+			return str(helpers.get_global_status())
+		# return show for the given account
+		status = gajim.connections[account].status
+		return DBUS_STRING(status)
+		
+
+	def get_account_and_contact(self, account, jid):
+		''' get the account (if not given) and contact instance from jid'''
+		connected_account = None
+		contact = None
+		accounts = gajim.contacts.get_accounts()
+		# if there is only one account in roster, take it as default
+		# if user did not ask for account
+		if not account and len(accounts) == 1:
+			account = accounts[0]
+		if account:
+			if gajim.connections[account].connected > 1: # account is connected
+				connected_account = account
+				contact = gajim.contacts.get_contact_with_highest_priority(account,
+					jid)
+		else:
+			for account in accounts:
+				contact = gajim.contacts.get_contact_with_highest_priority(account,
+					jid)
+				if contact and gajim.connections[account].connected > 1:
+					# account is connected
+					connected_account = account
+					break
+		if not contact:
+			contact = jid
+		
+		return connected_account, contact
 
 	def send_file(self, *args):
 		'''send_file(file_path, jid, account=None) 
 		send file, located at 'file_path' to 'jid', using account 
 		(optional) 'account' '''
 		file_path, jid, account = self._get_real_arguments(args, 3)
-		accounts = gajim.contacts.keys()
-		
-		# if there is only one account in roster, take it as default
-		# if user did not ask for account
-		if not account and len(accounts) == 1:
-			account = accounts[0]
-		if account:
-			if gajim.connections[account].connected > 1: # account is  online
-				connected_account = gajim.connections[account]
-		else:
-			for account in accounts:
-				if gajim.contacts[account].has_key(jid) and \
-					gajim.connections[account].connected > 1: # account is  online
-					connected_account = gajim.connections[account]
-					break
-		if gajim.contacts.has_key(account) and \
-			gajim.contacts[account].has_key(jid):
-			contact = gajim.get_highest_prio_contact_from_contacts(
-				gajim.contacts[account][jid])
-		else:
-			contact = jid
-		
+
+		connected_account, contact = self.get_account_and_contact(account, jid)
+
 		if connected_account:
 			if os.path.isfile(file_path): # is it file?
-				gajim.interface.instances['file_transfers'].send_file(account, 
-					contact, file_path)
+				gajim.interface.instances['file_transfers'].send_file(
+					connected_account, contact, file_path)
 				return True
 		return False
-		
+
 	def send_message(self, *args):
 		''' send_message(jid, message, keyID=None, account=None)
 		send 'message' to 'jid', using account (optional) 'account'.
@@ -181,23 +259,12 @@ class SignalObject(DbusPrototype):
 			return None # or raise error
 		if not keyID:
 			keyID = ''
-		connected_account = None
-		accounts = gajim.contacts.keys()
 		
-		# if there is only one account in roster, take it as default
-		if not account and len(accounts) == 1:
-			account = accounts[0]
-		if account:
-			if gajim.connections[account].connected > 1: # account is  online
-				connected_account = gajim.connections[account]
-		else:
-			for account in accounts:
-				if gajim.contacts[account].has_key(jid) and \
-					gajim.connections[account].connected > 1: # account is  online
-					connected_account = gajim.connections[account]
-					break
+		connected_account, contact = self.get_account_and_contact(account, jid)
+		
 		if connected_account:
-			res = connected_account.send_message(jid, message, keyID)
+			connection = gajim.connections[connected_account]
+			res = connection.send_message(jid, message, keyID)
 			return True
 		return False
 
@@ -210,7 +277,7 @@ class SignalObject(DbusPrototype):
 			return None
 		if jid.startswith('xmpp:'):
 			jid = jid[5:] # len('xmpp:') = 5
-
+		
 		if account:
 			accounts = [account]
 		else:
@@ -221,15 +288,16 @@ class SignalObject(DbusPrototype):
 		first_connected_acct = None
 		for acct in accounts:
 			if gajim.connections[acct].connected > 1: # account is  online
-				if gajim.interface.instances[acct]['chats'].has_key(jid):
+				contact = gajim.contacts.get_first_contact_from_jid(acct, jid)
+				if gajim.interface.msg_win_mgr.has_window(jid, acct):
 					connected_account = acct
 					break
 				# jid is in roster
-				elif gajim.contacts[acct].has_key(jid):
+				elif contact:
 					connected_account = acct
 					break
-				# we send the message to jid not in roster, because account is specified,
-				# or there is only one account
+				# we send the message to jid not in roster, because account is
+				# specified, or there is only one account
 				elif account: 
 					connected_account = acct
 				elif first_connected_acct is None:
@@ -242,12 +310,12 @@ class SignalObject(DbusPrototype):
 		if connected_account:
 			gajim.interface.roster.new_chat_from_jid(connected_account, jid)
 			# preserve the 'steal focus preservation'
-			win = gajim.interface.instances[connected_account]['chats'][jid].window
+			win = gajim.interface.msg_win_mgr.get_window(jid, connected_account).window
 			if win.get_property('visible'):
 				win.window.focus()
 			return True
 		return False
-	
+		
 	def change_status(self, *args, **keywords):
 		''' change_status(status, message, account). account is optional -
 		if not specified status is changed for all accounts. '''
@@ -261,75 +329,87 @@ class SignalObject(DbusPrototype):
 				status, message)
 		else:
 			# account not specified, so change the status of all accounts
-			for acc in gajim.contacts.keys():
+			for acc in gajim.contacts.get_accounts():
 				gobject.idle_add(gajim.interface.roster.send_status, acc, 
 					status, message)
 		return None
-
+	
 	def show_next_unread(self, *args):
 		''' Show the window(s) with next waiting messages in tabbed/group chats. '''
 		#FIXME: when systray is disabled this method does nothing.
 		if len(gajim.interface.systray.jids) != 0:
 			gajim.interface.systray.handle_first_event()
-
+	
 	def contact_info(self, *args):
-		''' get vcard info for a contact. This method returns nothing.
-		You have to register the 'VcardInfo' signal to get the real vcard. '''
+		''' get vcard info for a contact. Return cached value of the vcard.
+		'''
 		[jid] = self._get_real_arguments(args, 1)
 		if not isinstance(jid, unicode):
 			jid = unicode(jid)
 		if not jid:
 			# FIXME: raise exception for missing argument (0.3+)
 			return None
-
-		accounts = gajim.contacts.keys()
+	
+		cached_vcard = gajim.connections.values()[0].get_cached_vcard(jid)
+		if cached_vcard:
+			return get_dbus_struct(cached_vcard)
 		
-		for account in accounts:
-			if gajim.contacts[account].__contains__(jid):
-				self.vcard_account =  account
-				gajim.connections[account].request_vcard(jid)
-				break
-		return None
-
+		# return empty dict
+		return DBUS_DICT_SV()
+	
 	def list_accounts(self, *args):
 		''' list register accounts '''
-		if gajim.contacts:
-			result = gajim.contacts.keys()
-			if result and len(result) > 0:
-				result_array = []
-				for account in result:
-					result_array.append(account.encode('utf-8'))
-				return result_array
+		result = gajim.contacts.get_accounts()
+		if result and len(result) > 0:
+			result_array = []
+			for account in result:
+				result_array.append(DBUS_STRING(account))
+			return result_array
 		return None
-
+	
+	def account_info(self, *args):
+		''' show info on account: resource, jid, nick, prio, message '''
+		[for_account] = self._get_real_arguments(args, 1)
+		if not gajim.connections.has_key(for_account):
+			# account is invalid
+			return None
+		account = gajim.connections[for_account]
+		result = DBUS_DICT_SS()
+		index = account.connected
+		result['status'] = DBUS_STRING(STATUS_LIST[index])
+		result['name'] = DBUS_STRING(account.name)
+		result['jid'] = DBUS_STRING(gajim.get_jid_from_account(account.name))
+		result['message'] = DBUS_STRING(account.status)
+		result['priority'] = DBUS_STRING(unicode(gajim.config.get_per('accounts', 
+								account.name, 'priority')))
+		result['resource'] = DBUS_STRING(unicode(gajim.config.get_per('accounts', 
+								account.name, 'resource')))
+		return result
+	
 	def list_contacts(self, *args):
 		''' list all contacts in the roster. If the first argument is specified,
 		then return the contacts for the specified account '''
 		[for_account] = self._get_real_arguments(args, 1)
 		result = []
-		if not gajim.contacts or len(gajim.contacts) == 0:
+		accounts = gajim.contacts.get_accounts()
+		if len(accounts) == 0:
 			return None
 		if for_account:
-			if gajim.contacts.has_key(for_account):
-				for jid in gajim.contacts[for_account]:
-					item = self._serialized_contacts(
-						gajim.contacts[for_account][jid])
-					if item:
-						result.append(item)
-			else:
-				# 'for_account: is not recognised:', 
-				return None
+			accounts_to_search = [for_account]
 		else:
-			for account in gajim.contacts:
-				for jid in gajim.contacts[account]:
-					item = self._serialized_contacts(gajim.contacts[account][jid])
+			accounts_to_search = accounts
+		for account in accounts_to_search:
+			if account in accounts:
+				for jid in gajim.contacts.get_jid_list(account):
+					item = self._contacts_as_dbus_structure(
+						gajim.contacts.get_contact(account, jid))
 					if item:
 						result.append(item)
 		# dbus 0.40 does not support return result as empty list
 		if result == []:
 			return None
 		return result
-
+	
 	def toggle_roster_appearance(self, *args):
 		''' shows/hides the roster window '''
 		win = gajim.interface.roster.window
@@ -344,7 +424,7 @@ class SignalObject(DbusPrototype):
 				win.window.focus(long(time()))
 
 	def prefs_list(self, *args):
-		prefs_dict = {}
+		prefs_dict = DBUS_DICT_SS()
 		def get_prefs(data, name, path, value):
 			if value is None:
 				return
@@ -353,9 +433,9 @@ class SignalObject(DbusPrototype):
 				for node in path:
 					key += node + "#"
 			key += name
-			prefs_dict[key] = unicode(value[1])
+			prefs_dict[DBUS_STRING(key)] = DBUS_STRING(value[1])
 		gajim.config.foreach(get_prefs)
-		return repr(prefs_dict)
+		return prefs_dict
 		
 	def prefs_store(self, *args):
 		try:
@@ -376,7 +456,7 @@ class SignalObject(DbusPrototype):
 		else:
 			gajim.config.del_per(key_path[0], key_path[1], key_path[2])
 		return True
-		
+	
 	def prefs_put(self, *args):
 		[key] = self._get_real_arguments(args, 1)
 		if not key:
@@ -389,33 +469,33 @@ class SignalObject(DbusPrototype):
 		subname, value = key_path[2].split('=', 1)
 		gajim.config.set_per(key_path[0], key_path[1], subname, value)
 		return True
-		
+	
 	def add_contact(self, *args):
 		[account] = self._get_real_arguments(args, 1)
-		if gajim.contacts.has_key(account):
+		accounts = gajim.contacts.get_accounts()
+		if account in accounts:
 			AddNewContactWindow(account)
 			return True
 		return False
 	
 	def remove_contact(self, *args):
 		[jid, account] = self._get_real_arguments(args, 2)
-		accounts = gajim.contacts.keys()
+		accounts = gajim.contacts.get_accounts()
 		
 		# if there is only one account in roster, take it as default
 		if account:
 			accounts = [account]
-		else:
-			accounts = gajim.contacts.keys()
 		contact_exists = False
 		for account in accounts:
-			if gajim.contacts[account].has_key(jid):
+			contacts = gajim.contacts.get_contact(account, jid)
+			if contacts:
 				gajim.connections[account].unsubscribe(jid)
-				for contact in gajim.contacts[account][jid]:
+				for contact in contacts:
 					gajim.interface.roster.remove_contact(contact, account)
-				del gajim.contacts[account][jid]
+				gajim.contacts.remove_jid(account, jid)
 				contact_exists = True
 		return contact_exists
-		
+	
 	def _is_first(self):
 		if self.first_show:
 			self.first_show = False
@@ -423,28 +503,31 @@ class SignalObject(DbusPrototype):
 		return False
 
 	def _get_real_arguments(self, args, desired_length):
-		# supresses the first 'message' argument, which is set in dbus 0.23
-		if dbus_support.version[1] == 20:
-			args=args[1:]
+		''' extend, or descend the length of args to match desired_length 
+		'''
+		args = list(args)
+		for i in range(len(args)):
+			if args[i]: 
+				args[i] = unicode(args[i])
+			else:
+				args[i] = None
 		if desired_length > 0:
-			args = list(args)
 			args.extend([None] * (desired_length - len(args)))
 			args = args[:desired_length]
 		return args
-
-	def _serialized_contacts(self, contacts):
-		''' get info from list of Contact objects and create a serialized
-		dict for sending it over dbus '''
+	
+	def _contacts_as_dbus_structure(self, contacts):
+		''' get info from list of Contact objects and create dbus dict '''
 		if not contacts:
 			return None
 		prim_contact = None # primary contact
 		for contact in contacts:
 			if prim_contact == None or contact.priority > prim_contact.priority:
 				prim_contact = contact
-		contact_dict = {}
-		contact_dict['name'] = prim_contact.name
-		contact_dict['show'] = prim_contact.show
-		contact_dict['jid'] = prim_contact.jid
+		contact_dict = DBUS_DICT_SV()
+		contact_dict['name'] = DBUS_VARIANT(DBUS_STRING(prim_contact.name))
+		contact_dict['show'] = DBUS_VARIANT(DBUS_STRING(prim_contact.show))
+		contact_dict['jid'] = DBUS_VARIANT(DBUS_STRING(prim_contact.jid))
 		if prim_contact.keyID:
 			keyID = None
 			if len(prim_contact.keyID) == 8:
@@ -455,36 +538,40 @@ class SignalObject(DbusPrototype):
 				contact_dict['openpgp'] = keyID
 		contact_dict['resources'] = []
 		for contact in contacts:
-			contact_dict['resources'].append(tuple([contact.resource, 
-				contact.priority, contact.status]))
-		return repr(contact_dict)
-	
-	
+			resource_props = [DBUS_STRING(contact.resource), contact.priority, DBUS_STRING(contact.status)]
+			contact_dict['resources'].append(tuple(resource_props))
+		contact_dict['resources'] = DBUS_VARIANT(contact_dict['resources'])
+		return contact_dict
+
+	def get_unread_msgs_number(self, *args):
+		return str(gajim.interface.roster.nb_unread)
+
 	if dbus_support.version[1] >= 30 and dbus_support.version[1] <= 40:
 		method = dbus.method
 		signal = dbus.signal
 	elif dbus_support.version[1] >= 41:
 		method = dbus.service.method
 		signal = dbus.service.signal
-
-	if dbus_support.version[1] >= 30:
-		# prevent using decorators, because they are not supported 
-		# on python < 2.4
-		# FIXME: use decorators when python2.3 (and dbus 0.23) is OOOOOOLD
-		toggle_roster_appearance = method(INTERFACE)(toggle_roster_appearance)
-		list_contacts = method(INTERFACE)(list_contacts)
-		list_accounts = method(INTERFACE)(list_accounts)
-		show_next_unread = method(INTERFACE)(show_next_unread)
-		change_status = method(INTERFACE)(change_status)
-		open_chat = method(INTERFACE)(open_chat)
-		contact_info = method(INTERFACE)(contact_info)
-		send_message = method(INTERFACE)(send_message)
-		send_file = method(INTERFACE)(send_file)
-		VcardInfo = signal(INTERFACE)(VcardInfo)
-		prefs_list = method(INTERFACE)(prefs_list)
-		prefs_put = method(INTERFACE)(prefs_put)
-		prefs_del = method(INTERFACE)(prefs_del)
-		prefs_store = method(INTERFACE)(prefs_store)
-		remove_contact = method(INTERFACE)(remove_contact)
-		add_contact = method(INTERFACE)(add_contact)
-		get_status = method(INTERFACE)(get_status)
+	
+	# prevent using decorators, because they are not supported 
+	# on python < 2.4
+	# FIXME: use decorators when python2.3 (and dbus 0.23) is OOOOOOLD
+	toggle_roster_appearance = method(INTERFACE)(toggle_roster_appearance)
+	list_contacts = method(INTERFACE)(list_contacts)
+	list_accounts = method(INTERFACE)(list_accounts)
+	show_next_unread = method(INTERFACE)(show_next_unread)
+	change_status = method(INTERFACE)(change_status)
+	open_chat = method(INTERFACE)(open_chat)
+	contact_info = method(INTERFACE)(contact_info)
+	send_message = method(INTERFACE)(send_message)
+	send_file = method(INTERFACE)(send_file)
+	prefs_list = method(INTERFACE)(prefs_list)
+	prefs_put = method(INTERFACE)(prefs_put)
+	prefs_del = method(INTERFACE)(prefs_del)
+	prefs_store = method(INTERFACE)(prefs_store)
+	remove_contact = method(INTERFACE)(remove_contact)
+	add_contact = method(INTERFACE)(add_contact)
+	get_status = method(INTERFACE)(get_status)
+	get_status_message = method(INTERFACE)(get_status_message)
+	account_info = method(INTERFACE)(account_info)
+	get_unread_msgs_number = method(INTERFACE)(get_unread_msgs_number)
